#ifndef SEARCHTASK_H
#define SEARCHTASK_H

#include <QObject>
#include <QReadWriteLock>

#include <UkuiSearchTask>

#include "condition.h"
#include "file/fileinfo.h"

class QReadWriteLock;

class SearchTask : public QObject
{
    friend class SearchResultModel;
    friend class ModelController;
    Q_OBJECT
public:
    explicit SearchTask(QObject *parent = nullptr);
    ~SearchTask();

    const QVariant &getConditionsDescription() const;
    void setConditionsDescription(const QVariant &newConditionsDescription);

signals:
    void conditionsDescriptionChanged();
    void searchFinished(bool successed);
    void searchContentFinished(bool successed);
    void finished();

public slots:
    void start();
    void cancel();

protected:
    void handleSearchResult();

private:
    UkuiSearch::UkuiSearchTask *ukuiSearchTask;
    UkuiSearch::DataQueue<UkuiSearch::ResultItem> *queue;
    UkuiSearch::UkuiSearchTask *ukuiSearchContentTask;
    UkuiSearch::DataQueue<UkuiSearch::ResultItem> *contentQueue;
    QList<QSharedPointer<UKUI::IDM::FileInfo>> resultInfos;
    QList<QSharedPointer<UKUI::IDM::FileInfo>> contentResultInfos;

    bool needSearchFile = false;
    bool needSearchFileContent = false;
    bool hasContentSearchAndCondition = false;

    bool searchDone = false;
    bool searchContentDone = false;

    QReadWriteLock rwlock;
    QVariant conditionsDescription;
    QList<QSharedPointer<Condition>> conditions;
    Q_PROPERTY(QVariant conditionsDescription READ getConditionsDescription WRITE setConditionsDescription NOTIFY conditionsDescriptionChanged)
};

#endif // SEARCHTASK_H
