#ifndef FILEINFO_H
#define FILEINFO_H

#include <QSharedPointer>
#include <QVariant>

namespace UKUI {
namespace IDM {

class FileInfoPrivate;

class FileInfo
{
    friend class FileInfoJob;
public:
    static QSharedPointer<FileInfo> fromPath(const QString &filePath, bool addWatch = true);
    static QSharedPointer<FileInfo> fromUri(const QString &uri, bool addWatch = true);
    static QSharedPointer<FileInfo> fromUrl(const QUrl &url, bool addWatch = true);

    bool isValid() const;
    QString uri() const;
    QString path() const;
    QString parentPath() const;
    QString displayName() const;
    QString iconName() const;
    QString dateString() const;
    QString version() const;
    QString mimeType() const;
    QString fileType() const;
    quint64 timeModifiedMsec() const;
    quint64 timeAccessMsec() const;
    quint64 size() const;
    QString sizeFormat() const;
    QList<QVariant> labelColors() const;

    ~FileInfo();

private:
    FileInfo();

    FileInfoPrivate *priv;
};
}
}

#endif // FILEINFO_H
