#ifndef FILEINFOJOB_H
#define FILEINFOJOB_H

#include <QObject>
#include <QRunnable>

#include "fileinfo.h"

class Condition;

namespace UKUI {
namespace IDM {
class FileInfoJobPrivate;

class FileInfoJob : public QObject, public QRunnable
{
    Q_OBJECT
public:
    static FileInfoJob *fromUris(const QStringList &uris, bool addWatchByDefault = true);
    static FileInfoJob *fromPaths(const QStringList &paths, bool addWatchByDefault = true);
    static FileInfoJob *fromInfos(const QList<QSharedPointer<UKUI::IDM::FileInfo>> &infos);
    ~FileInfoJob();

    void start();
    void cancel();

    /*!
     * \brief setConditions
     * 设置搜索过滤条件
     * \param conditions
     */
    void setConditions(const QList<QSharedPointer<Condition>> &conditions);

    void run() override;

    QList<QSharedPointer<FileInfo>> getInfos();

signals:
    void started();
    void finished();
    void cancelled();
    void errored(const QString &errorMessage);
    void infoUpdated(const QSharedPointer<FileInfo> &info);

private:
    explicit FileInfoJob(const QStringList &args, bool isUriFormat, bool addWatchByDefault = true, QObject *parent = nullptr);
    FileInfoJob();

    FileInfoJobPrivate *priv;
};

#endif // FILEINFOJOB_H

}
}
