#ifndef FILEINFOMANAGER_H
#define FILEINFOMANAGER_H

#include <QObject>

#include "fileinfo.h"

namespace UKUI {
namespace IDM {

class FileInfoManagerPrivate;

class FileInfoManager : public QObject
{
    Q_OBJECT
public:
    enum FileEventType {
        Unknown,
        FileCreated,
        FileModified,
        FileDeleted,
        FileMoveOut,
        FileMoveIn,
        FileAccessed,
        FileOpened,
        FileClosed,
        FileAttrChanged,
        FileIgnoredAutomatic,
        FileUnmount,
        UserAddFile,
        UserRemoveFile,
        Other
    };
    Q_ENUM(FileEventType)

    static FileInfoManager *globalInstance();

    void addFileToWatcher(const QString &url, bool *ok = nullptr);
    QStringList addFilesToWatcher(const QStringList &urls);
    void removeFileFromWatcher(const QString &url, bool *ok = nullptr);
    QStringList removeFilesFromWatcher(const QStringList &urls);

Q_SIGNALS:
    void fileChanged(FileEventType type, const QString &arg1, const QString arg2);
    void remoteFileEvent(int eventType, const QString &filePath);

private:
    FileInfoManagerPrivate *priv;
    explicit FileInfoManager(QObject *parent = nullptr);
    ~FileInfoManager();

    friend class FileInfo;
    QSharedPointer<FileInfo> getInfoByUri(const QString &uri);
    void insertInfo(QSharedPointer<FileInfo> info);
};

}
}

#endif // FILEINFOMANAGER_H
