#ifndef FILEITEMPROXYSORTFILTERMODEL_H
#define FILEITEMPROXYSORTFILTERMODEL_H

#include <QSortFilterProxyModel>

#include "fileitemmodel.h"

class QCollator;

class FileItemProxySortFilterModel : public QSortFilterProxyModel
{
    Q_OBJECT
    Q_PROPERTY(SortType sortType READ sortType WRITE setSortType)
    Q_PROPERTY(Qt::SortOrder sortOrder READ sortOrder)
    Q_PROPERTY(QString spaceName READ spaceName NOTIFY spaceNameChanged)
public:
    enum SortType {
        MostUsed,
        AddMode, //手动添加优先，自动添加优先两种模式
        FileName,
        Date, // 创建时间，修改时间，访问时间？
        FileType,
        FileSize
    };
    Q_ENUM(SortType)

    explicit FileItemProxySortFilterModel(FileItemModel *sourceModel, QObject *parent = nullptr);
    explicit FileItemProxySortFilterModel(QObject *parent = nullptr);
    ~FileItemProxySortFilterModel();

    SortType sortType() const;
    void setSortType(const SortType &sortType);

    Q_INVOKABLE void doSort(int sortType, int order, bool userAddedFirst);

    Q_INVOKABLE void startLoadRecentFiles();
    Q_INVOKABLE void startLoadFiles();

    Q_INVOKABLE int getSpaceId() const;
    Q_INVOKABLE QString getSpaceName() const;

    Q_INVOKABLE QStringList getSpaceFilePaths() const;

    Q_INVOKABLE QString getSpaceStatusDescription();

    QString spaceName();

    void handleFileEvent(UKUI::IDM::FileInfoManager::FileEventType type, const QString &arg1, const QString arg2);

    bool isUserAddedFile(const QString &path);

signals:
    void filesLoaded();
    void requestStart();
    void spaceNameChanged();
    void spacesStatusDescriptionChanged();

public slots:
    void handleSpaceRemoved(int spaceId);

protected:
    bool lessThan(const QModelIndex &source_left, const QModelIndex &source_right) const;

    QString getDisplayName(const QModelIndex &index) const;
    bool isDirectory(const QModelIndex &index) const;
    quint64 getModificationTime(const QModelIndex &index) const;
    QString getFileMimeType(const QModelIndex &index) const;
    quint64 getFileSize(const QModelIndex &index) const;
    QString getFileSizeFormat(const QModelIndex &index) const;
    quint64 getFileAccessTime(const QModelIndex &index) const;

    quint64 getLastAddedFileCount() const;
    quint64 getLastAddedFileTimeMsec() const;

private:
    SortType m_sortType = MostUsed;

    bool m_userAddedFirst = true;

    QCollator *m_collator = nullptr;

    FileItemModel *m_sourceModel = nullptr;
};

#endif // FILEITEMPROXYSORTFILTERMODEL_H
