#ifndef MODELCONTROLLER_H
#define MODELCONTROLLER_H

#include <QObject>
#include <QUrl>

#include "fileitemproxysortfiltermodel.h"

class SpacesFilesFetcher;
class Condition;
class FileItemProxySortFilterModel;

class ModelController : public QObject
{
    friend class PreviewHelper;
    Q_OBJECT
public:
    static ModelController *globalInstance();

    explicit ModelController(const QString &dbName = "ukui-idm.db", QObject *parent = nullptr);
    ~ModelController();

    Q_INVOKABLE void startLoadModel();
    Q_INVOKABLE void startLoadRecentModel();
    Q_INVOKABLE void openFile(const QString &path);
    Q_INVOKABLE void userAddFilesToSpace(const QList<QUrl> &urls, FileItemProxySortFilterModel *model, bool sendSignal = true);
    Q_INVOKABLE void removeFileFromSpace(const QString &path, FileItemProxySortFilterModel *model, bool sendSignal = true);
    Q_INVOKABLE void removeFileFromAllSpace(const QString &path);
    Q_INVOKABLE void editSpaceCondition(int spaceId);
    Q_INVOKABLE void createSpace();
    Q_INVOKABLE void removeSpace(int spaceId);
    Q_INVOKABLE void requestShowMenu(FileItemProxySortFilterModel *model, int index, const QPoint &globalPos);
    Q_INVOKABLE void editSearch();
    Q_INVOKABLE void saveSearchResultAsSpace();
    Q_INVOKABLE void requestAddUserAddedFileToSapce(FileItemProxySortFilterModel *model);

    void renameSpace(int spaceId, const QString &spaceName);

    QMap<int, QString> getSpaceIdsWithSpace();

    QList<QSharedPointer<UKUI::IDM::FileInfo>> getUserAddedFilesFromSpaceById(int spaceId);

    FileItemProxySortFilterModel *getProxyModelBySpaceId(int spaceId);

public slots:
    void onSpaceConditionChanged(int spaceId, const QString &spaceName, const QVariant conditionsDescription);
    void onSpaceCreateRequest(const QString &spaceName, const QVariant &conditionDescription, bool isCreatedFromSearch);
    void onSpaceCreateRequestWithFiles(const QString &spaceName, const QVariant &conditionDescription, bool isCreatedFromSearch, QList<QSharedPointer<UKUI::IDM::FileInfo>> infos);

signals:
    void modelCreated(FileItemProxySortFilterModel *model);
    void recentModelCreated(FileItemProxySortFilterModel *model);
    void allModelCreated();

    void setSpaceConditionRequest(int spaceId, const QString &spaceName);
    void createSpaceRequest();

    void spaceNameUpdated(int spaceId, const QString &spaceName);
    void spaceCreated(int spaceId, const QString &spaceName);
    void spaceRemoved(int spaceId);
    void spaceConditionsChanged(int spaceId, const QString &spaceName, const QVariant &conditionsDescription);
    void showMenuRequested(FileItemProxySortFilterModel *model, int index, const QPoint &globalPos);
    void editSearchRequested();
    void searchSaveRequested();

    // 用于和条件过滤线程交互
    void fileCreateRequest(const QString &newFile);
    void spaceConditionChangedRequest(int spaceId, const QList<QSharedPointer<Condition>> &conditions);

    // 从线程返回的结果
    void filesCreated(const QList<QSharedPointer<UKUI::IDM::FileInfo>> &infos, int spaceId);
    void spaceFilesChangedForConditionsChanged(bool needReload, const QList<QSharedPointer<UKUI::IDM::FileInfo>> &resultFiles, int spaceId);

    // 空间条件改变返回的结果
    void filesReloaded(const QList<QSharedPointer<UKUI::IDM::FileInfo>> &infos, int spaceId);

private slots:
    void initModels();
    void handleRescanSpaceWithCondition(int spaceId, const QVariant &conditionsDescription);

private:
    SpacesFilesFetcher *fetcher;
    QThread *thread;
    QMap<int, FileItemProxySortFilterModel *> proxyModels;
};

#endif // MODELCONTROLLER_H
