#include "previewhelper.h"

#include "modelcontroller.h"
#include "spacesfilesfetcher.h"

PreviewHelper *instance = nullptr;

PreviewHelper *PreviewHelper::globalInstance()
{
    if (!instance) {
        instance = new PreviewHelper;
    }
    return instance;
}

PreviewHelper::PreviewHelper(QObject *parent)
    : QObject{parent}
{

}

void PreviewHelper::setPreviewFile(const QString &path)
{
    fileInfo = UKUI::IDM::FileInfo::fromPath(path, false);

    setFilePath(path);
    setDisplayName(fileInfo->displayName());
    setAccessTime(fileInfo->dateString());
    setModificationTime(fileInfo->dateString());
    setFileSize(fileInfo->sizeFormat());
}

void PreviewHelper::setPreviewSpaceId(int spaceId)
{
    fileInfo = nullptr;

    auto spaceName = ModelController::globalInstance()->fetcher->getSpaceNameById(spaceId);
    setFilePath(nullptr);
    setDisplayName(spaceName);

    setAccessTime(nullptr);
    setModificationTime(nullptr);
    setFileSize(nullptr);
}

const QString &PreviewHelper::getDisplayName() const
{
    return displayName;
}

void PreviewHelper::setDisplayName(const QString &newDisplayName)
{
    if (displayName == newDisplayName)
        return;
    displayName = newDisplayName;
    emit displayNameChanged();
}

const QString &PreviewHelper::getAccessTime() const
{
    return accessTime;
}

void PreviewHelper::setAccessTime(const QString &newAccessTime)
{
    if (accessTime == newAccessTime)
        return;
    accessTime = newAccessTime;
    emit accessTimeChanged();
}

const QString &PreviewHelper::getModificationTime() const
{
    return modificationTime;
}

void PreviewHelper::setModificationTime(const QString &newModificationTime)
{
    if (modificationTime == newModificationTime)
        return;
    modificationTime = newModificationTime;
    emit modificationTimeChanged();
}

const QString &PreviewHelper::getFileSize() const
{
    return fileSize;
}

void PreviewHelper::setFileSize(const QString &newFileSize)
{
    if (fileSize == newFileSize)
        return;
    fileSize = newFileSize;
    emit fileSizeChanged();
}

const QString &PreviewHelper::getFilePath() const
{
    return filePath;
}

void PreviewHelper::setFilePath(const QString &newFilePath)
{
    if (filePath == newFilePath)
        return;
    filePath = newFilePath;
    emit filePathChanged();
}
