#ifndef PREVIEWHELPER_H
#define PREVIEWHELPER_H

#include <QObject>

#include "../file/fileinfo.h"

class PreviewHelper : public QObject
{
    Q_OBJECT
public:
    static PreviewHelper *globalInstance();
    explicit PreviewHelper(QObject *parent = nullptr);

    Q_INVOKABLE void setPreviewFile(const QString &path);
    Q_INVOKABLE void setPreviewSpaceId(int spaceId);

    const QString &getDisplayName() const;
    void setDisplayName(const QString &newDisplayName);

    const QString &getAccessTime() const;
    void setAccessTime(const QString &newAccessTime);

    const QString &getModificationTime() const;
    void setModificationTime(const QString &newModificationTime);

    const QString &getFileSize() const;
    void setFileSize(const QString &newFileSize);

    const QString &getFilePath() const;
    void setFilePath(const QString &newFilePath);

signals:
    void displayNameChanged();

    void accessTimeChanged();

    void modificationTimeChanged();

    void fileSizeChanged();

    void filePathChanged();

private:
    bool isFileMode = true;

    QString filePath;
    QString displayName;
    QString accessTime;
    QString modificationTime;
    QString fileSize;
    Q_PROPERTY(QString displayName READ getDisplayName WRITE setDisplayName NOTIFY displayNameChanged)
    Q_PROPERTY(QString accessTime READ getAccessTime WRITE setAccessTime NOTIFY accessTimeChanged)
    Q_PROPERTY(QString modificationTime READ getModificationTime WRITE setModificationTime NOTIFY modificationTimeChanged)
    Q_PROPERTY(QString fileSize READ getFileSize WRITE setFileSize NOTIFY fileSizeChanged)

    QSharedPointer<UKUI::IDM::FileInfo> fileInfo;
    Q_PROPERTY(QString filePath READ getFilePath WRITE setFilePath NOTIFY filePathChanged)
};

#endif // PREVIEWHELPER_H
