#ifndef SPACESFILESFETCHER_H
#define SPACESFILESFETCHER_H

#include <QObject>
#include <QMap>
#include <QReadWriteLock>

namespace UKUI {
namespace IDM {
class FileInfo;
}
}

class Condition;
class QTimer;

class SpacesFilesFetcher : public QObject
{
    friend class ModelController;
    friend class PreviewHelper;
    Q_OBJECT
public:
    explicit SpacesFilesFetcher(const QString &dbName = "ukui-idm.db", QObject *parent = nullptr);

    QStringList getAllSpacesName();
    int getSpaceIdByName(const QString &spaceName);
    QString getSpaceNameById(int spaceId);

    QList<QSharedPointer<UKUI::IDM::FileInfo>> getFilesInSpace(const QString &spaceName, bool isUserAdded = false);
    QList<QSharedPointer<UKUI::IDM::FileInfo>> getFilesInSpaceById(int spaceId, bool isUserAdded = false);

signals:
    void fetched(bool successed);

    void handleSpaceConditionsChangedDone(int spaceId);
    void handleFilesCreatedProgressed(const QList<QSharedPointer<UKUI::IDM::FileInfo>> &newFiles, int spaceId);
    void handleSpaceConditionsProgressed(bool needReload, const QList<QSharedPointer<UKUI::IDM::FileInfo>> &resultFiles, int spaceId);

    void requestRescanSpaceWithConditionsDescritpion(int spaceId, const QVariant &conditionsDescription);
    void requestClearSpace(int spaceId);

public slots:
    void fetch();
    void updateSpaceConditions(int spaceId, const QString &spaceName, const QVariant &conditionsDescription);
    void handleFileCreated(const QString &newFile);
    void handleAfterFilesReloaded(const QList<QSharedPointer<UKUI::IDM::FileInfo>> &infos, int spaceId);
    void handleSpaceRemoved(int spaceId);

protected slots:
    void handleFilesCreatedInternal();

private:
    QReadWriteLock rwLock;

    QTimer *delayer = nullptr;
    QStringList createdFilesToHandle;

    QMap<int, QString> spaces;
    QMap<int, QList<QSharedPointer<Condition>>> spacesConditions;
    QMap<int, QList<QSharedPointer<UKUI::IDM::FileInfo>>> infos;
    QMap<int, QList<QSharedPointer<UKUI::IDM::FileInfo>>> userAddedInfos;

    QString dbName;
};

#endif // SPACESFILESFETCHER_H
