#include <QtTest>
#include <QObject>
#include <QDir>
#include <QStandardPaths>

#include "fileitemmodel.h"
#include "fileitemproxysortfiltermodel.h"
#include "modelcontroller.h"
#include "databasemanager.h"

class ModelTest : public QObject
{
    Q_OBJECT
public:
    ModelTest() {}

public Q_SLOTS:
    void onModelCreated(FileItemProxySortFilterModel *model);

private Q_SLOTS:
    void initTestCase();
    void test1ControllerInit();
    void test2ModelInit();
    void test3RecentModel();
    void cleanupTestCase();

private:
    ModelController *controller;
    QList<FileItemProxySortFilterModel *> models;
};

QTEST_MAIN(ModelTest)
#include "modeltest.moc"

void ModelTest::onModelCreated(FileItemProxySortFilterModel *model)
{
    models << model;
}

void ModelTest::initTestCase()
{
    QDir d(QStandardPaths::writableLocation(QStandardPaths::DataLocation));
    d.mkpath(d.absolutePath());
    DatabaseManager manager("modeltest.db");
    auto spaces = manager.getAllSpaceNames();
    d.cd("/bin");
    QStringList paths;
    for (auto entry : d.entryList()) {
        paths << d.filePath(entry);
    }
    for (auto space : spaces) {
        for (auto file : paths) {
            manager.addFileToSpace(file, space);
        }
        // UserAddedFile空间
        manager.addFileToSpace("/home", space, true);
    }
}

void ModelTest::test1ControllerInit()
{
    controller = new ModelController("modeltest.db");
    connect(controller, &ModelController::modelCreated, this, &ModelTest::onModelCreated);
    QEventLoop l;
    connect(controller, &ModelController::allModelCreated, &l, &QEventLoop::quit);
    controller->startLoadModel();
    l.exec();
}

void ModelTest::test2ModelInit()
{
    QVERIFY(models.count() > 0);
    for (auto model : models) {
        QEventLoop l;
        connect(model, &FileItemProxySortFilterModel::filesLoaded, &l, &QEventLoop::quit);
        model->startLoadFiles();
        l.exec();
        qDebug() << model->getSpaceId() << model->getSpaceName();
        qDebug() << model->rowCount();
        qDebug() << model->index(model->rowCount() - 1, 0).data();
    }
}

void ModelTest::test3RecentModel()
{
    FileItemProxySortFilterModel proxyModel(new FileItemModel);
    QEventLoop loop;
    connect(&proxyModel, &FileItemProxySortFilterModel::filesLoaded, this, [&]{
        qDebug() << proxyModel.rowCount();
        loop.quit();
    });
    proxyModel.startLoadRecentFiles();
    loop.exec();
}

void ModelTest::cleanupTestCase()
{
    for (auto model : models) {
        delete model;
    }
    delete controller;
    QDir d(QStandardPaths::writableLocation(QStandardPaths::DataLocation));
    d.removeRecursively();
}
