#include <QtTest>
#include <QObject>
#include <QDir>
#include <QImage>
#include <QStandardPaths>

#include "../thumbnail/thumbnailmanager.h"

class ThumbnailTest : public QObject
{
    Q_OBJECT
public:
    ThumbnailTest() {}

private Q_SLOTS:
    void initTestCase();
    void test1Thumbnail();
    void test2ThumbnailAgain();
    void cleanupTestCase();

private:
    ThumbnailManager *manager;
};

QTEST_MAIN(ThumbnailTest)
#include "thumbnailtest.moc"

void ThumbnailTest::initTestCase()
{
    QDir dir("/tmp");
    Q_ASSERT(dir.mkdir("ukui-idm-thumbnailtest"));
    dir.cd("ukui-idm-thumbnailtest");
    QImage image(QSize(500, 500), QImage::Format_ARGB32_Premultiplied);
    image.fill(Qt::red);
    for (auto i = 0; i < 100; i++) {
        image.save(dir.absoluteFilePath(QString("%1.png").arg(i)));
    }

    manager = new ThumbnailManager;
}

void ThumbnailTest::test1Thumbnail()
{
    QEventLoop *loop = new QEventLoop;
    QDir dir("/tmp/ukui-idm-thumbnailtest");
    QStringList entries = dir.entryList(QDir::Files);
    QTimeLine timeLine;
    timeLine.setLoopCount(1);
    timeLine.setDuration(100 * 100);
    timeLine.setFrameRange(0, 100);
    connect(manager, &ThumbnailManager::thumbnailFinished, loop, [=, &entries](const QString &path, const QImage &image, const QString &errorMessage){
        if (entries.isEmpty()) {
            loop->quit();
            delete loop;
        }
        qDebug() << path << image.size();
        QVERIFY(image.size().isValid());
    });
    connect(&timeLine, &QTimeLine::frameChanged, this, [=, &entries]{
        if (entries.isEmpty())
            return;
        auto name = entries.takeFirst();
        QString path = dir.absoluteFilePath(name);
        manager->startPreview(path);
    });
    timeLine.start();
    loop->exec();
}

void ThumbnailTest::test2ThumbnailAgain()
{
    manager->setStrictedMode(true);
    test1Thumbnail();
}

void ThumbnailTest::cleanupTestCase()
{
    QDir dir("/tmp/ukui-idm-thumbnailtest");
    dir.removeRecursively();

    delete manager;
}
