import QtQuick 2.0
import QtQuick.Controls 2.0
import QtQuick.Layouts 1.12

import org.ukui.quick.items 1.0
import org.ukui.quick.platform 1.0 as Platform
import ThumbnailManager 1.0
import PreviewHelper 1.0

Item {
    property bool isSpace: false
    property bool opened: false
    property int rootSpacing: 24
    property int controlSpacing: 8
    property string displayName: previewHelper.displayName

    visible: mainWindow.currentView.spaceId > 0

    anchors.fill: parent

    Item {
        id: rootItem
        anchors.fill: parent
        anchors.margins: rootSpacing

        ThemeIcon {
            id: previewIcon
            width: parent.width
            height: parent.width * 0.534
            anchors.top: parent.top
            //source: "window-close-symbolic"
        }

        Text {
            id: nameLabel
            Layout.maximumWidth: parent.width/2
            anchors.top: previewIcon.bottom
            anchors.left: parent.left
            anchors.topMargin: rootSpacing
            text: displayName
        }

        ComboBox {
            id: relatedFilesBox
            Layout.maximumWidth: parent.width/2
            anchors.right: parent.right
            anchors.verticalCenter: nameLabel.verticalCenter
            anchors.topMargin: rootSpacing
            visible: nameLabel.visible && opened
            model : [qsTr("Related Files"), "1", "2", "3"]
        }

        ThemeSeparator {
            id: separator1
            anchors.top: relatedFilesBox.bottom
            anchors.topMargin: controlSpacing
            width: parent.width
        }

        DisabledText {
            anchors.top: separator1.bottom
            anchors.topMargin: controlSpacing
            id: locationText
            text: qsTr("File Location")
        }

        DisabledText {
            Layout.maximumWidth: parent.width/2
            anchors.verticalCenter: locationText.verticalCenter
            wrapMode: Text.WrapAtWordBoundaryOrAnywhere
            horizontalAlignment: Text.AlignRight
            anchors.right: parent.right
            anchors.left: locationText.right
            height: locationText.height
            text: previewHelper.filePath
            visible: locationText.visible && opened
        }

        DisabledText {
            id: accessTimeText
            anchors.top: locationText.bottom
            anchors.topMargin: controlSpacing
            text: isSpace? qsTr("Create Time"): qsTr("Access Time")
        }

        DisabledText {
            anchors.verticalCenter: accessTimeText.verticalCenter
            wrapMode: Text.WrapAtWordBoundaryOrAnywhere
            anchors.right: parent.right
            text: previewHelper.accessTime
            visible: accessTimeText.visible && opened
        }

        DisabledText {
            id: modifiedTime
            anchors.top: accessTimeText.bottom
            anchors.topMargin: controlSpacing
            text: qsTr("Modification Time")
        }

        DisabledText {
            anchors.verticalCenter: modifiedTime.verticalCenter
            wrapMode: Text.WrapAtWordBoundaryOrAnywhere
            anchors.right: parent.right
            text: previewHelper.modificationTime
            visible: modifiedTime.visible && opened
        }

        DisabledText {
            id: sizeText
            visible: !isSpace
            anchors.top: modifiedTime.bottom
            anchors.topMargin: controlSpacing
            text: qsTr("File Size")
        }

        DisabledText {
            anchors.verticalCenter: sizeText.verticalCenter
            wrapMode: Text.WrapAtWordBoundaryOrAnywhere
            anchors.right: parent.right
            text: previewHelper.fileSize
            visible: sizeText.visible  && opened
        }

        DisabledText {
            id: sourceText
            visible: !isSpace
            anchors.top: sizeText.bottom
            anchors.topMargin: controlSpacing
            text: qsTr("File Source")
        }

        DisabledText {
            anchors.verticalCenter: sourceText.verticalCenter
            wrapMode: Text.WrapAtWordBoundaryOrAnywhere
            anchors.right: parent.right
            //fixme: file source
            visible: sourceText.visible && opened
        }

        ThemeSeparator {
            id: separator2
            width: parent.width
            anchors.top: isSpace? modifiedTime.bottom: sourceText.bottom
            anchors.topMargin: controlSpacing
        }

        LinkButton {
            id: openButton
            anchors.top: separator2.bottom
            anchors.topMargin: controlSpacing
            text: isSpace? qsTr("Edit Conditions"): qsTr("Open File")

            onTriggered: {
                if (isSpace) {
                    modelController.editSpaceCondition(mainWindow.currentView.spaceId);
                } else {
                    modelController.openFile(previewHelper.filePath);
                }
            }
        }

        LinkButton {
            id: openDirectoryButton
            anchors.top: openButton.bottom
            anchors.topMargin: controlSpacing
            text: isSpace? qsTr("Rename"): qsTr("Open File Directory")

            onTriggered: {
                if (isSpace) {
                    modelController.editSpaceCondition(mainWindow.currentView.spaceId);
                } else {
                    modelController.openFile(previewHelper.filePath);
                }
            }
        }

        LinkButton {
            id: removeSpaceButton
            anchors.top: openDirectoryButton.bottom
            anchors.topMargin: controlSpacing
            text: qsTr("Delete Space")
            visible: isSpace

            onTriggered: {
                if (mainWindow.currentView.spaceId >= 0)
                    spaceTabBar.currentItem.spaceRemoveRequest(mainWindow.currentView.spaceId);
            }
        }

        ThumbnailManager {
            id: thumbnailManager
            onThumbnailFinished: {
                console.log(path, image, errorMessage);
                if (errorMessage === "")
                    previewIcon.source = image;
                else
                    console.log(errorMessage);
            }
        }
    }

    Button {
        id: aiAssistantButton
        x: parent.x + rootSpacing - width/2
        y: parent.y + rootSpacing - height/2
        width: 36
        height: 36

        ThemeIcon {
            anchors.fill: parent
            source: "preview"
        }
    }

    function startPreviewFile(path, decoration) {
        if (!visible)
            return;

        isSpace = false;

        // todo: use ukui file metadata api for generating thumbnail
        previewIcon.source = decoration;
        thumbnailManager.setStrictedMode(true);
        thumbnailManager.startPreview(path);

        previewHelper.setPreviewFile(path);
    }

    function startPreviewSpace(spaceId) {
        if (!visible)
            return;

        isSpace = true;
        // fixme: display space icon
        previewIcon.source = "folder";

        previewHelper.setPreviewSpaceId(spaceId);
    }

    onVisibleChanged: {
        if (visible) {
            startPreviewSpace(mainWindow.currentView.spaceId);
        } else {
            previewIcon.source = "";
        }
    }
}
