import QtQuick 2.0
import QtQuick.Controls 2.0 as QQC2
import QtQuick.Controls 1.4 as QQC1
import QtQuick.Layouts 1.0

import org.ukui.quick.items 1.0 as UKUIItems
import org.ukui.quick.platform 1.0 as UKUIPlatform

import SearchResultProxyModel 1.0
import QWindow 1.0

Item {
    property string keyword: searchEdit.text
    property bool searchInSpace: false
    property FileItemView lastSearchView: (mainWindow.modelInited && swipeView.count > 0)? mainWindow.currentView: emptyView
    property int controlSpacing: 8
    property QQC1.Menu spaceMenu: spaceSelectionMenu
    property string currentSpace: (mainWindow.modelInited && swipeView.count > 0)? mainWindow.currentView.viewSpaceName: ""

    onSearchInSpaceChanged: {
        searchModel.setSearchInSpacePaths(lastSearchView.proxyModel.getSpaceFilePaths());
        searchInComputerButton.checked = swipeView.count > 0? !searchInSpace: true;
        searchInSpaceButton.checked = swipeView.count > 0? searchInSpace: false;
        searchModel.searchInSpace = searchInSpace;
    }

    onLastSearchViewChanged: {
        searchModel.setSearchInSpacePaths(lastSearchView.proxyModel.getSpaceFilePaths());
        searchModel.currentSpaceId = lastSearchView.spaceId;
    }

    onKeywordChanged: {
        searchModel.searchKeyword = keyword;
    }

    Item {
        id: searchHeader
        anchors.top: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.margins: controlSpacing
        height: 48

        Row {
            id: searchLocationButtons
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter

            QQC1.Button {
                id: searchInComputerButton
                height: 36
                text: qsTr("Computer")

                checkable: true
                checked: swipeView.count > 0? !searchInSpace: true

                onClicked: {
                    searchInSpace = false;
                }
            }

            QQC1.Button {
                id: searchInSpaceButton
                height: 36
                text: currentSpace
                menu: spaceSelectionMenu
                visible: swipeView.count > 0

                checkable: true
                checked: swipeView.count > 0? searchInSpace: false

                QQC1.Menu {
                    id: spaceSelectionMenu
                }
            }
        }

        Row {
            id: optionButtons
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter

            QQC1.Button {
                height: 36
                text: qsTr("Save as Space")

                visible: !searchInSpace

                onClicked: {
                    modelController.saveSearchResultAsSpace();
                }
            }
        }

        LinkButton {
            anchors.right: optionButtons.left
            anchors.rightMargin: controlSpacing
            anchors.verticalCenter: optionButtons.verticalCenter
            text: qsTr("Edit Condition")

            onTriggered: {
                modelController.editSearch();
            }
        }
    }

    Item {
        id: searchConditionSubLine
        anchors.top: searchHeader.bottom
        anchors.topMargin: controlSpacing
        anchors.left: parent.left
        anchors.leftMargin: controlSpacing
        anchors.right: parent.right
        anchors.rightMargin: controlSpacing
        height: 30

        Text {
            id: searchSettingsLabel
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
            text: qsTr("Search Condition Settings")
        }

        LinkButton {
            anchors.left: searchSettingsLabel.right
            anchors.leftMargin: controlSpacing
            anchors.verticalCenter: parent.verticalCenter
            text: qsTr("Edit Condition")

            onTriggered: {
                modelController.editSearch();
            }
        }
    }

    Item {
        anchors.top: searchConditionSubLine.bottom
        anchors.topMargin: controlSpacing
        anchors.left: parent.left
        anchors.leftMargin: controlSpacing
        anchors.right: parent.right
        anchors.rightMargin: controlSpacing
        anchors.bottom: parent.bottom
        anchors.bottomMargin: controlSpacing

        QQC1.TreeView {
            id: searchResultView
            anchors.fill: parent
            backgroundVisible: false
            QQC1.TableViewColumn {
                title: "File Name"
                role: "display"
                width: parent.width - 236 * 2 - controlSpacing * 2
            }
            QQC1.TableViewColumn {
                title: "File Size"
                role: "displaySize"
                width: 236
                visible: !previewPageVisible || mainWindow.visibility === QWindow.Maximized
            }
            QQC1.TableViewColumn {
                title: "Time Info"
                role: "displayTime"
                width: 236
                visible: !previewPageVisible || mainWindow.visibility === QWindow.Maximized
            }

            model: searchModel

            onClicked: {
                var icon = model.data(currentIndex, Qt.DecorationRole);
                var path = model.data(currentIndex, Qt.UserRole + 1);
                console.log("tree view clicked", path);
                previewPage.startPreviewFile(path, icon);
            }

            Component.onCompleted: {
                searchModel.modelReset.connect(function (){
                    searchResultView.expand(model.index(0, 0));
                    searchResultView.expand(model.index(1, 0));
                });
            }
        }
    }
}
