import QtQuick 2.0
import QtQuick.Window 2.0
import QtQuick.Controls 2.12
import QtQuick.Layouts 1.12
import QtQuick.Controls 1.4
//import SqliteModels 1.0
import ModelController 1.0
import FileItemModel 1.0
import FileItemProxySortFilterModel 1.0
import SearchResultProxyModel 1.0
import QWindow 1.0

ApplicationWindow {
    id: mainWindow
    property bool previewPageVisible: false
    property color windowBg: "transparent"
    property FileItemView currentView: modelInited && swipeView.count > 0? swipeView.currentItem: emptyView
    property int expandedMode: 3
    property int expanderHeight: 48
    property int expandDuration: 200
    property int controlSpacing: 8
    property bool modelInited: false
    signal expandedChanged()
    signal viewTypeChanged()
    signal sortTypeChanged()
    signal systemMoveRequest(point pos)
    visible: true
    width: 1096
    height: 688
    minimumWidth: 800
    minimumHeight: 600
    title: qsTr("Intelligent Data Manager")

    flags: Qt.WA_TranslucentBackground|Qt.WA_NoSystemBackground
    color: windowBg

    FileItemView {
        id: emptyView
    }

    onPreviewPageVisibleChanged: {
        hSplitView.state = previewPageVisible? "previewOpened": "previewClosed"
        //previewPageContainer.visible = previewPageVisible;
        //previewPage.visible = previewPageVisible;
        previewPage.opened = previewPageVisible;
    }

    onExpandedChanged: {
        var i = 0;
        if (fileItemViewExpander.expanded) {
            i = i + 1;
        }
        if (recentExpander.expanded) {
            i = i + 2;
        }

        mainWindow.expandedMode = i;
        console.log("expanded change", mainWindow.expandedMode, fileItemViewExpander.expanded, recentExpander.expanded);
        switch (mainWindow.expandedMode) {
        case 0: {
            mainContentView.state = "allCollapsed";
            break;
        }
        case 1: {
            mainContentView.state = "spaceViewExpanded";
            break;
        }
        case 2: {
            mainContentView.state = "recentViewExpanded";
            break;
        }
        case 3: {
            mainContentView.state = "allExpanded";
            break;
        }
        default:
            mainContentView.state = "allExpanded";
        }
    }

    onCurrentViewChanged: {
        console.log(searchPage.currentSpace);
        previewPage.startPreviewSpace(currentView.spaceId);
    }

    ThemeBackground {
        anchors.fill: parent
    }

    ColumnLayout {
        anchors.fill: parent
        spacing: 0
        ThemeBackground {
            id: headerBarContainer
            isTransparent: false
            Layout.fillWidth: true
            height: 56

            MouseArea {
                property bool pressed: false
                anchors.fill: parent
                hoverEnabled: true
                propagateComposedEvents: true
                acceptedButtons: Qt.LeftButton

                onPressed: {
                    mouse.accepted = true;
                    pressed = true;
                }

                onReleased: {
                    mouse.accepted = true;
                    console.log("mouse released");
                    pressed = false;
                }

                onPositionChanged: {
                    //console.log("mouse position changed", mouse.x, mouse.y);
                    mouse.accepted = true;
                    if (pressed) {
                        var pos = headerBarContainer.mapToGlobal(mouse.x, mouse.y);
                        systemMoveRequest(pos);
                        //mainWindow.startSystemMove();
                        pressed = false;
                    }
                }
            }

            RowLayout {
                anchors.fill: parent
                anchors.leftMargin: controlSpacing

                Image {
                    id: headerIcon
                    Layout.alignment: Qt.AlignLeft
                    Layout.preferredWidth: 20
                    Layout.preferredHeight: 20
                    Layout.leftMargin: controlSpacing
                    verticalAlignment: Qt.AlignVCenter
                    source: "qrc:/data/ukui-idm.png"
                }

                Text {
                    id: headerTitle
                    Layout.alignment: Qt.AlignLeft
                    Layout.fillHeight: true
                    verticalAlignment: Qt.AlignVCenter
                    text: mainWindow.title
                }

                TextField {
                    id: searchEdit
                    Layout.alignment: Qt.AlignCenter
                    Layout.maximumWidth: 480
                    Layout.fillWidth: true
                    placeholderText: qsTr("Search")
                    horizontalAlignment: focus? Qt.AlignLeft: Qt.AlignHCenter;

                    Component.onCompleted: {
                        searchModel.setSearchKeywordReqeust.connect(function (keyword){
                            searchEdit.text = keyword;
                        });
                    }
                }

                StyledToolBar {
                    id: headerButtons
                    z: 999
                    Layout.fillHeight: true
                    Layout.rightMargin: controlSpacing
                    Layout.alignment: Qt.AlignRight

                    RowLayout {
                        height: parent.height

                        StyledButton {
                            Layout.preferredWidth: 30
                            Layout.preferredHeight: 30
                            width: 30
                            height: 30
                            flat: true
                            state: "WindowButton"
                            iconName: "ukui-preview-file-symbolic"

                            visible: mainWindow.currentView.spaceId > 0

                            onClicked: {
                                previewPageVisible = !previewPageVisible;
                            }
                        }

                        ToolSeparator {
                            visible: mainWindow.currentView.spaceId > 0
                        }

                        StyledButton {
                            Layout.preferredWidth: 30
                            Layout.preferredHeight: 30
                            width: 30
                            height: 30
                            flat: true
                            state: "WindowButton"
                            iconName: "open-menu-symbolic"
                        }

                        StyledButton {
                            Layout.preferredWidth: 30
                            Layout.preferredHeight: 30
                            width: 30
                            height: 30
                            flat: true
                            state: "WindowButton"
                            iconName: "window-minimize-symbolic"

                            onClicked: {
                                mainWindow.showMinimized();
                            }
                        }

                        StyledButton {
                            Layout.preferredWidth: 30
                            Layout.preferredHeight: 30
                            width: 30
                            height: 30
                            flat: true
                            state: "WindowButton"
                            iconName: mainWindow.visibility === QWindow.Maximized? "window-restore-symbolic": "window-maximize-symbolic"

                            onClicked: {
                                if (mainWindow.visibility === QWindow.Maximized) {
                                    mainWindow.showNormal();
                                } else {
                                    mainWindow.showMaximized();
                                }
                            }
                        }

                        StyledButton {
                            Layout.preferredWidth: 30
                            Layout.preferredHeight: 30
                            width: 30
                            height: 30
                            iconName: "window-close-symbolic"
                            flat: true
                            state: "CloseButton"

                            onClicked: mainWindow.close();
                        }
                    }
                }
            }
        }

        SplitView {
            id: hSplitView
            Layout.fillWidth: true
            Layout.fillHeight: true
            //anchors.fill: parent

            state: "previewClosed"

            states: [
                State {
                    name: "previewClosed"
                    PropertyChanges {
                        target: previewPageContainer
                        width: 0
                    }
                },
                State {
                    name: "previewOpened"
                    PropertyChanges {
                        target: previewPageContainer
                        width: parent.width * 1/3
                    }
                }
            ]

            transitions: [
                Transition {
                    from: "previewClosed"
                    to: "previewOpened"
                    NumberAnimation {
                        target: previewPageContainer
                        property: "width"
                        duration: expandDuration
                        easing.type: Easing.InOutQuad
                    }
                },
                Transition {
                    from: "previewOpened"
                    to: "previewClosed"
                    NumberAnimation {
                        target: previewPageContainer
                        property: "width"
                        duration: expandDuration
                        easing.type: Easing.InOutQuad
                    }
                }
            ]

            Item {
                Layout.fillWidth: true
                width: parent.width * 2/3

                SearchPage {
                    id: searchPage
                    width: parent.width
                    height: parent.height
                    anchors.left: parent.left
                    state: searchEdit.text.length? "shown": "hidden"

                    states: [
                        State {
                            name: "hidden"
                            PropertyChanges {
                                target: searchPage
                                y: -parent.height
                                opacity: 0
                            }
                        },
                        State {
                            name: "shown"
                            PropertyChanges {
                                target: searchPage
                                y: 0
                                opacity: 1.0
                            }
                        }
                    ]

                    transitions: [
                        Transition {
                            from: "hidden"
                            to: "shown"

                            NumberAnimation {
                                target: searchPage
                                property: "y"
                                duration: 500
                                easing.type: Easing.InOutQuad
                            }

                            NumberAnimation {
                                target: searchPage
                                property: "opacity"
                                duration: 500
                                easing.type: Easing.InQuad
                            }
                        },
                        Transition {
                            from: "shown"
                            to: "hidden"

                            NumberAnimation {
                                target: searchPage
                                property: "y"
                                duration: 500
                                easing.type: Easing.InOutQuad
                            }

                            NumberAnimation {
                                target: searchPage
                                property: "opacity"
                                duration: 500
                                easing.type: Easing.OutQuad
                            }
                        }
                    ]
                }

                Item {
                    id: mainPage

                    width: parent.width
                    height: parent.height
                    anchors.left: parent.left
                    state: searchEdit.text.length? "hidden": "shown"

                    states: [
                        State {
                            name: "hidden"
                            PropertyChanges {
                                target: mainPage
                                y: -parent.height
                                opacity: 0
                            }
                        },
                        State {
                            name: "shown"
                            PropertyChanges {
                                target: mainPage
                                y: 0
                                opacity: 1.0
                            }
                        }
                    ]

                    transitions: [
                        Transition {
                            from: "hidden"
                            to: "shown"

                            NumberAnimation {
                                target: mainPage
                                property: "y"
                                duration: 500
                                easing.type: Easing.InOutQuad
                            }

                            NumberAnimation {
                                target: mainPage
                                property: "opacity"
                                duration: 500
                                easing.type: Easing.InQuad
                            }
                        },
                        Transition {
                            from: "shown"
                            to: "hidden"

                            NumberAnimation {
                                target: mainPage
                                property: "y"
                                duration: 500
                                easing.type: Easing.InOutQuad
                            }

                            NumberAnimation {
                                target: mainPage
                                property: "opacity"
                                duration: 500
                                easing.type: Easing.OutQuad
                            }
                        }
                    ]

                    SplitView {
                        id: mainContentView
                        orientation: Qt.Vertical
                        anchors.left: parent.left
                        width: parent.width
                        height: parent.height
                        state: "allExpanded"

                        onStateChanged: {
                            console.log(mainContentView.state);
                        }

                        states: [
                            State {
                                name: "allCollapsed"
                                PropertyChanges {
                                    target: itemViewContainer
                                    height: expanderHeight
                                }
                                PropertyChanges {
                                    target: recentViewContainer
                                    height: expanderHeight
                                }
                            },
                            State {
                                name: "spaceViewExpanded"
                                PropertyChanges {
                                    target: itemViewContainer
                                    height: parent.height - expanderHeight
                                }
                                PropertyChanges {
                                    target: recentViewContainer
                                    height: expanderHeight
                                }
                            },
                            State {
                                name: "recentViewExpanded"
                                PropertyChanges {
                                    target: itemViewContainer
                                    height: expanderHeight
                                }
                                PropertyChanges {
                                    target: recentViewContainer
                                    height: parent.height - expanderHeight
                                }
                            },
                            State {
                                name: "allExpanded"
                                PropertyChanges {
                                    target: itemViewContainer
                                    height: parent.height * 2/3
                                }
                                PropertyChanges {
                                    target: recentViewContainer
                                    height: parent.height * 1/3
                                }
                            }
                        ]

                        transitions: [
                            Transition {
                                from: "allExpanded"
                                to: "spaceViewExpanded"

                                NumberAnimation {
                                    targets: [itemViewContainer, recentViewContainer]
                                    properties: "height"
                                    duration: expandDuration
                                }
                            },
                            Transition {
                                from: "spaceViewExpanded"
                                to: "allCollapsed"

                                NumberAnimation {
                                    targets: [itemViewContainer, recentViewContainer]
                                    properties: "height"
                                    duration: expandDuration
                                }
                            },
                            Transition {
                                from: "allCollapsed"
                                to: "spaceViewExpanded"

                                NumberAnimation {
                                    targets: [itemViewContainer, recentViewContainer]
                                    properties: "height"
                                    duration: expandDuration
                                }
                            },
                            Transition {
                                from: "spaceViewExpanded"
                                to: "allExpanded"

                                NumberAnimation {
                                    targets: [itemViewContainer, recentViewContainer]
                                    properties: "height"
                                    duration: expandDuration
                                }
                            },

                            Transition {
                                from: "allExpanded"
                                to: "recentViewExpanded"

                                NumberAnimation {
                                    targets: [itemViewContainer, recentViewContainer]
                                    properties: "height"
                                    duration: expandDuration
                                }
                            },
                            Transition {
                                from: "recentViewExpanded"
                                to: "allCollapsed"

                                NumberAnimation {
                                    targets: [itemViewContainer, recentViewContainer]
                                    properties: "height"
                                    duration: expandDuration
                                }
                            },
                            Transition {
                                from: "allCollapsed"
                                to: "recentViewExpanded"

                                NumberAnimation {
                                    targets: [itemViewContainer, recentViewContainer]
                                    properties: "height"
                                    duration: expandDuration
                                }
                            },
                            Transition {
                                from: "recentViewExpanded"
                                to: "allExpanded"

                                NumberAnimation {
                                    targets: [itemViewContainer, recentViewContainer]
                                    properties: "height"
                                    duration: expandDuration
                                }
                            }
                        ]

                        Item {
                            id: itemViewContainer
                            //Layout.fillHeight: true
                            height: parent.height*2/3

                            Item {
                                id: fileItemViewExpander
                                property bool expanded: true
                                anchors.top: parent.top
                                anchors.left: parent.left
                                anchors.right: parent.right
                                height: expanderHeight

                                RowLayout {
                                    anchors.fill: parent
                                    anchors.leftMargin: mainWindow.controlSpacing
                                    anchors.rightMargin: mainWindow.controlSpacing
                                    spacing: mainWindow.controlSpacing
                                    Text {
                                        id: fileItemViewExpanderText
                                        verticalAlignment: Qt.AlignVCenter
                                        text: fileItemViewExpander.expanded? qsTr("Collapse Space"): qsTr("Expand Space")

                                        MouseArea {
                                            anchors.fill: parent
                                            onClicked: {
                                                fileItemViewExpander.expanded = !fileItemViewExpander.expanded;
                                                console.log("item view expanded changed:", fileItemViewExpander.expanded);
                                                expandedChanged()
                                            }
                                        }
                                    }

                                    SpaceTabBar {
                                        id: spaceTabBar
                                        visible: count > 0
                                        clip: true
                                        currentIndex: 0
                                        Layout.maximumWidth: parent.width - fileItemViewExpanderText.width - createSpaceTabButton.width - sortMenuButton.width - viewTypeButton.width - 6*mainWindow.controlSpacing
                                    }

                                    Button {
                                        id: createSpaceTabButton
                                        visible: spaceTabBar.visible
                                        Layout.preferredWidth: 36
                                        Layout.preferredHeight: 36
                                        iconName: "list-add-symbolic"
                                        onClicked: {
                                            console.log("create space");
                                            modelController.createSpace();
                                        }
                                    }

                                    Rectangle {
                                        color: "transparent"
                                        Layout.fillWidth: true
                                    }

                                    Button {
                                        id: sortMenuButton
                                        visible: spaceTabBar.visible
                                        Layout.preferredWidth: 60
                                        Layout.preferredHeight: 36
                                        iconName: "view-sort-ascending-symbolic"

                                        menu: Menu {
                                            MenuItem {
                                                text: qsTr("Most Used")
                                                onTriggered: {
                                                    console.log("most used triggered", swipeView.currentIndex);
                                                    mainWindow.currentView.sortRequest(0, -1, true);
                                                }
                                            }

                                            Menu {
                                                title: qsTr("Add Mode")
                                                MenuItem {
                                                    text: qsTr("User Added")
                                                    onTriggered: {
                                                        console.log("user added triggered");
                                                        mainWindow.currentView.sortRequest(1, -1, true);
                                                    }
                                                }
                                                MenuItem {
                                                    text: qsTr("Auto Added")
                                                    onTriggered: {
                                                        console.log("auto added triggered");
                                                        mainWindow.currentView.sortRequest(1, -1, false);
                                                    }
                                                }
                                            }

                                            MenuItem {
                                                text: qsTr("File Name")
                                                onTriggered: {
                                                    console.log("file name triggered");
                                                    mainWindow.currentView.sortRequest(2, -1, true);
                                                }
                                            }

                                            MenuItem {
                                                text: qsTr("Date")
                                                onTriggered: {
                                                    console.log("date triggered");
                                                    mainWindow.currentView.sortRequest(3, -1, true);
                                                }
                                            }

                                            MenuItem {
                                                text: qsTr("File Type")
                                                onTriggered: {
                                                    console.log("file type triggered");
                                                    mainWindow.currentView.sortRequest(4, -1, true);
                                                }
                                            }

                                            MenuItem {
                                                text: qsTr("File Size")
                                                onTriggered: {
                                                    console.log("file size triggered");
                                                    mainWindow.currentView.sortRequest(5, -1, true);
                                                }
                                            }

                                            MenuSeparator {}

                                            MenuItem {
                                                text: qsTr("Ascending Order")
                                                onTriggered: {
                                                    console.log("ascend triggered");
                                                    currentView.sortRequest(-1, 0, true);
                                                }
                                            }

                                            MenuItem {
                                                text: qsTr("Descending Order")
                                                onTriggered: {
                                                    console.log("descend triggered");
                                                    currentView.sortRequest(-1, 1, true);
                                                }
                                            }

                                            MenuSeparator {}

                                            MenuItem {
                                                text: qsTr("Use Global Sorting")
                                                onTriggered: {
                                                    console.log("global sorting triggered");
                                                }
                                            }
                                        }
                                    }

                                    Button {
                                        id: viewTypeButton
                                        visible: spaceTabBar.visible
                                        Layout.preferredWidth: 36
                                        Layout.preferredHeight: 36
                                        property bool isIconMode: true

                                        iconName: isIconMode? "view-grid-symbolic": "view-list-symbolic"

                                        onClicked: {
                                            isIconMode = !isIconMode;
                                            viewTypeChanged();
                                            //mainWindow.currentView.switchView(isIconMode? 1: 2);
                                        }
                                    }
                                }
                            }

                            SwipeView {
                                id: swipeView
                                currentIndex: modelInited? spaceTabBar.currentIndex: spaceTabBar.count - 1
                                anchors.top: fileItemViewExpander.bottom
                                anchors.left: parent.left
                                anchors.right: parent.right
                                anchors.bottom: parent.bottom

                                onCurrentIndexChanged: {
                                    if (modelInited) {
                                        spaceTabBar.currentIndex = currentIndex;
                                    }
                                }
                            }

                            PageIndicator {
                                id: indicator
                                visible: fileItemViewExpander.expanded

                                count: swipeView.count
                                currentIndex: swipeView.currentIndex

                                anchors.bottom: swipeView.bottom
                                anchors.horizontalCenter: parent.horizontalCenter
                            }
                        }

                        Item {
                            id: recentViewContainer
                            //Layout.fillHeight: true
                            height: parent.height*1/3

                            Item {
                                property bool expanded: true
                                id: recentExpander
                                anchors.top: parent.top
                                anchors.left: parent.left
                                anchors.leftMargin: controlSpacing
                                anchors.right: parent.right
                                height: expanderHeight

                                Text {
                                    anchors.left: parent.left
                                    width: parent.width
                                    height: expanderHeight
                                    verticalAlignment: Qt.AlignVCenter
                                    color: "black"
                                    anchors.verticalCenter: parent.verticalCenter
                                    text: recentExpander.expanded? qsTr("Collapse Recent"): qsTr("Expand Recent")
                                }

                                MouseArea {
                                    anchors.fill: parent
                                    onPressed: {
                                        recentExpander.expanded = !recentExpander.expanded;
                                        console.log("expanded changed", recentExpander.expanded);
                                        expandedChanged()
                                    }
                                }
                            }

                            FileItemView {
                                anchors.top: recentExpander.bottom
                                anchors.left: parent.left
                                anchors.right: parent.right
                                anchors.bottom: parent.bottom
                                id: recentFileItemView
                                visible: mainWindow.expandedMode / 2 >= 1
                                acceptDrop: false
                                Component.onCompleted: {
                                    switchView(2);
                                    modelController.recentModelCreated.connect(function (model){
                                        console.log("recent model loaded");
                                        recentFileItemView.setModel(model);
                                        model.filesLoaded.connect(function (){
                                            console.log("recent files loaded");
                                        });
                                    });

                                    modelController.startLoadRecentModel();
                                }

                                onPreviewRequest: {
                                    console.log("request preview from recent item view", path, icon);
                                    previewPage.startPreviewFile(path, icon);
                                }

                                onOpenRequest: {
                                    console.log("request open from recent item view", path);
                                    modelController.openFile(path);
                                }

                                onMenuRequest: {
                                    console.log("recent view menu request");
                                    modelController.requestShowMenu(proxyModel, index, globalPos);
                                }
                            }
                        }
                    }
                }
            }

            Item {
                id: previewPageContainer
                //visible: previewPageVisible
                width: parent.width * 1/3
                anchors.margins: 10
                Rectangle {
                    anchors.fill: parent
                    anchors.margins: 10
                    color: headerBarContainer.styleBackgroundColor
                    radius: 6
                    PreviewPage {
                        id: previewPage
                        anchors.fill: parent
                    }
                }
            }
        }
    }

    NoSpacePage {
        anchors.fill: parent
        z: 999
        visible: modelInited? (swipeView.count == 0 && mainContentView.state != "recentViewExpanded"): false
    }

    Component.onCompleted: {
        modelController.allModelCreated.connect(function(){
            modelInited = true;
            console.log("modelInited space tab bar", spaceTabBar.count);
            if (spaceTabBar.count > 0) {
                // ugly
                spaceTabBar.currentIndex = -1;
                spaceTabBar.currentIndex = 0;
            }
        });
        modelController.modelCreated.connect(function(model){
            console.log("on model created", model.getSpaceName());
            console.debug(model.getSpaceName(), model.getSpaceId());

            model.filesLoaded.connect(function (){
                console.log("files loaded");
            });
            model.startLoadFiles();
            var component = Qt.createComponent("FileItemView.qml");
            if (component.status === Component.Ready) {
                var viewInstance = component.createObject(swipeView);
                viewInstance.setCollapse(previewPageVisible);
                viewInstance.setModel(model);
                if (model.getSpaceId() === 1) {
                    console.log("on model created bg red");
                    viewInstance.contentWidth = swipeView.width
                } else {
                    console.log("on model created bg blue");
                    viewInstance.contentWidth = swipeView.width
                }

                mainWindow.previewPageVisibleChanged.connect(function (){
                    console.log("previewPageVisible changed", previewPageVisible);
                    viewInstance.setCollapse(previewPageVisible);
                });

                swipeView.widthChanged.connect(function (){
                    viewInstance.contentWidth = swipeView.width
                });

                viewInstance.openRequest.connect(function (path){
                    console.log("request open from view", viewInstance, path);
                    modelController.openFile(path);
                });
                viewInstance.previewRequest.connect(function (path, decoration){
                    console.log("request preview from view", viewInstance, path, decoration);
                    previewPage.startPreviewFile(path, decoration);
                });
                viewInstance.previewSpaceRequest.connect(function (spaceId){
                    console.log("start preview space", spaceId);
                    previewPage.startPreviewSpace(spaceId);
                });
                viewInstance.dropRequest.connect(function (urls){
                    console.log("add files to space");
                    modelController.userAddFilesToSpace(urls, model);
                });
                viewInstance.sortRequest.connect(function (sortType, sortOrder, userAddedFirst){
                    console.log("sort", sortType, sortOrder, userAddedFirst);
                    model.doSort(sortType, sortOrder, userAddedFirst)
                });
                viewInstance.menuRequest.connect(function (proxyModel, index, globalPos){
                    if (viewInstance === mainWindow.currentView)
                        modelController.requestShowMenu(proxyModel, index, globalPos);
                });

                mainWindow.viewTypeChanged.connect(function (){
                    viewInstance.switchView(viewTypeButton.isIconMode? 1: 2);
                });

//                if (swipeView.index < 0) {
//                    console.log("init current index");
//                    mainWindow.currentView = viewInstance;
//                }

                viewInstance.switchView(viewTypeButton.isIconMode? 1: 2);
                swipeView.addItem(viewInstance);

                var menuItem = searchPage.spaceMenu.addItem(viewInstance.viewSpaceName);

                var tabButtonComponent = Qt.createComponent("SpaceTabButton.qml");
                console.log("tab button component status", tabButtonComponent.status, tabButtonComponent.errorString());
                if (tabButtonComponent.status === Component.Ready) {
                    var tabButton = tabButtonComponent.createObject(spaceTabBar);
                    tabButton.viewItem = viewInstance;
                    console.log("tab button created", tabButton, model.getSpaceName());
                    tabButton.text = model.getSpaceName();
                    tabButton.proxyModel = model;
                    spaceTabBar.addItem(tabButton);
                    viewInstance.tabButton = tabButton;

                    tabButton.spaceRenameRequest.connect(function (spaceId){
                        console.log("space rename request", spaceId);
                        modelController.editSpaceCondition(spaceId);
                    });
                    tabButton.spaceRemoveRequest.connect(function (spaceId){
                        console.log("space remove request", spaceId);
                        swipeView.removeItem(tabButton.viewItem);
                        spaceTabBar.removeTab(tabButton);
                        searchPage.spaceMenu.removeItem(menuItem);

                        delete viewInstance;
                        delete tabButton;
                        delete menuItem;
                        //delete model;

                        modelController.removeSpace(spaceId);
                    });
                    tabButton.spaceEditRequest.connect(function (spaceId){
                        console.log("space edit request", spaceId);
                        modelController.editSpaceCondition(spaceId);
                    });

                    menuItem.triggered.connect(function(){
                        console.log(tabButton);
                        searchPage.searchInSpace = true;
                        tabButton.toggle();
                    });

                    viewInstance.viewSpaceNameChanged.connect(function(){
                        menuItem.text = viewInstance.viewSpaceName;
                        tabButton.text = viewInstance.viewSpaceName;
                    });
                }
            } else {
                console.log("failed to create component", component.status, component.errorString());
            }
        });

        modelController.startLoadModel();
    }
}
