#include "windowhelper.h"

#include <QWindow>
#include <KWindowSystem>
#include <QX11Info>
#include <KWindowEffects>

#include <QDebug>

#include <xcb/xcb.h>
#include <netwm.h>
#include <xatom-helper.h>

WindowHelper::WindowHelper(QWindow *parent) : QObject(parent)
{
    rootWindow = parent;
}

void WindowHelper::handleSystemMoveRequest(QPointF pos)
{
    auto window = qobject_cast<QWindow *>(sender());

    qreal dpiRatio = window->devicePixelRatio();
    int nativeX = pos.x() * dpiRatio;
    int nativeY = pos.y() * dpiRatio;
    if (KWindowSystem::isPlatformX11()) {
        auto connection = QX11Info::connection();
        xcb_ungrab_pointer(connection, XCB_TIME_CURRENT_TIME);
        NETRootInfo(connection, NET::WMMoveResize).moveResizeRequest(window->winId(), nativeX, nativeY, NET::Move);

        xcb_button_release_event_t* event = new xcb_button_release_event_t;
        memset(event, 0x00, sizeof(xcb_button_release_event_t));
        event->response_type = XCB_BUTTON_RELEASE;
        event->event = window->winId();
        event->time = QX11Info::getTimestamp();
        event->same_screen = 1;
        event->root = QX11Info::appRootWindow();
        event->root_x = nativeX;
        event->root_y = nativeY;
        event->event_x = 0;
        event->event_y = 0;
        event->child = 0;
        event->state = 0;
        event->detail = XCB_BUTTON_INDEX_1;

        xcb_send_event(connection, false, window->winId(), XCB_EVENT_MASK_BUTTON_RELEASE, (char *) event);
        delete event;

        xcb_flush(connection);
    } else if (KWindowSystem::isPlatformWayland()) {
#if (QT_VERSION >= QT_VERSION_CHECK(5,15,0))
        window->startSystemMove();
#else
#endif
    }

    qDebug() << "movement" << pos;
}

void WindowHelper::activeWindow()
{
    qDebug() << rootWindow << rootWindow->winId();

    // fixme: adapt for wayland
    MotifWmHints hints;
    hints.flags = MWM_HINTS_FUNCTIONS|MWM_HINTS_DECORATIONS;
    hints.functions = MWM_FUNC_ALL;
    hints.decorations = MWM_DECOR_BORDER;
    XAtomHelper::getInstance()->setWindowMotifHint(rootWindow->winId(), hints);
    KWindowEffects::enableBlurBehind(rootWindow, true);
    rootWindow->show();
    KWindowSystem::forceActiveWindow(rootWindow->winId());
}

QWindow *WindowHelper::getRootWindow() const
{
    return rootWindow;
}
